@@ -1,4 +1,6 @@
 
+import sys
+
 import itertools
 
 import decimal
@@ -9,6 +11,10 @@ import pyrfc3339
 from .edn_lex import Keyword, Symbol
 from .edn_parse import TaggedElement
 
+if sys.version_info[0] == 3:
+    long = int
+    basestring = str
+    unicode = str
 
 def dump(obj):
     def seq(obj):
@@ -44,4 +50,4 @@ def dump(obj):
     elif isinstance(obj, TaggedElement):
         return str(obj)
     else:
-        raise NotImplementedError("Don't know how to handle {} : {}", type(obj), obj)
\ No newline at end of file
+        raise NotImplementedError("Don't know how to handle {} : {}", type(obj), obj)
