@@ -1,3 +1,6 @@
+from __future__ import print_function
+
+import sys
 
 import ply.lex
 import logging
@@ -5,6 +8,10 @@ import logging
 import re
 import decimal
 
+if sys.version_info[0] == 3:
+    long = int
+    basestring = str
+    unicode = str
 
 class BaseEdnType(object):
     def __init__(self, name):
@@ -20,6 +27,9 @@ class BaseEdnType(object):
     def __repr__(self):
         return "{}({})".format(self.__class__.__name__, self._name)
 
+    def __hash__(self):
+        return hash(self._name)
+
 
 class Keyword(BaseEdnType):
     def __str__(self):
@@ -49,9 +59,16 @@ tokens = ('WHITESPACE',
           'MAP_OR_SET_END',
           'TAG')
 
+PARTS = {}
+PARTS["non_nums"] = r"\w.*+!\-_?$%&=:#"
+PARTS["all"] = PARTS["non_nums"] + r"\d"
+PARTS["first"] = r"\w*!_?$%&="
+PARTS["special"] = r"\-+."
+PARTS["start"] = r"([{first}]|[{special}][{non_nums}]|[{special}])".format(**PARTS)
+SYMBOL = r"({start}[{all}]*\/[{all}]+|\/|{start}[{all}]*)".format(**PARTS)
+KEYWORD = r":([{all}]+\/[{all}]+|[{all}]+)".format(**PARTS)
+TAG = r"\#\w([{all}]*\/[{all}]+|[{all}]*)".format(**PARTS)
 
-SYMBOL_FIRST_CHAR = r'\w+!\-_$&=\.'
-SYMBOL = r"[{0}][{0}\d/]*".format(SYMBOL_FIRST_CHAR)
 t_VECTOR_START = '\['
 t_VECTOR_END = '\]'
 t_LIST_START = '\('
@@ -94,7 +111,7 @@ def t_NIL(t):
 
 
 def t_BOOLEAN(t):
-    r"""(true)|(false)"""
+    r"""(true|false)(?=([\s\])}]|$))"""
     if t.value == "false":
         t.value = False
     elif t.value == "true":
@@ -135,13 +152,13 @@ def t_DISCARD(t):
     pass # ignore
 
 
-@ply.lex.TOKEN(r'\#{}'.format(SYMBOL))
+@ply.lex.TOKEN(TAG)
 def t_TAG(t):
     t.value = t.value[1:]
     return t
 
 
-@ply.lex.TOKEN(":{}".format(SYMBOL))
+@ply.lex.TOKEN(KEYWORD)
 def t_KEYWORD(t):
     t.value = Keyword(t.value[1:])
     return t
@@ -154,9 +171,7 @@ def t_SYMBOL(t):
 
 
 def t_error(t):
-    print "Illegal character '%s'" % t.value[0]
-    t.lexer.skip(1)
-
+    raise SyntaxError("Illegal character '%s'" % t.value[0])
 
 def lex(text=None):
     kwargs = {}
@@ -165,4 +180,4 @@ def lex(text=None):
     l = ply.lex.lex(reflags=re.UNICODE, **kwargs)
     if text is not None:
         l.input(text)
-    return l
\ No newline at end of file
+    return l
